% -------------------------------------------------------------------------
% License GPL v3.0
% Author: Borja Ayerdi <borja.ayerdi@ehu.es>
% Computational Intelligence Group <www.ehu.es/ccwintco>
% University Of The Basque Country (UPV/EHU) 
% Use this at your own risk!
% -------------------------------------------------------------------------

function [salida] = testELM(TestingData, elm)

InputWeight = elm.InputWeight;
OutputWeight = elm.OutputWeight;
BiasofHiddenNeurons = elm.BiasofHiddenNeurons;
NumberofOutputNeurons = elm.NumberofOutputNeurons;
label = elm.label;
number_class= elm.number_class;

Elm_Type = 1;
TV.P=TestingData';
TV.T=round(rand(1,size(TestingData,1)));

%% LA PARTE DE TESTING
REGRESSION=0;
CLASSIFIER=1;
ActivationFunction = 'sig';
NumberofTestingData=size(TV.P,2);

%%%%%%%%% Processing the targets of testing
temp_TV_T=zeros(NumberofOutputNeurons, NumberofTestingData);
for i = 1:NumberofTestingData
    for j = 1:number_class
        if label(1,j) == TV.T(1,i)
            break; 
        end
    end
    temp_TV_T(j,i)=1;
end
TV.T=temp_TV_T*2-1;



%%%%%%%%%%% Calculate the output of testing input
start_time_test=cputime;
tempH_test=InputWeight*TV.P;
clear TV.P;             %   Release input of testing data             
ind=ones(1,NumberofTestingData);
BiasMatrix=BiasofHiddenNeurons(:,ind);              %   Extend the bias matrix BiasofHiddenNeurons to match the demention of H
tempH_test=tempH_test + BiasMatrix;


switch lower(ActivationFunction)
    case {'sig','sigmoid'}
        %%%%%%%% Sigmoid 
        H_test = 1 ./ (1 + exp(-tempH_test));
    case {'sin','sine'}
        %%%%%%%% Sine
        H_test = sin(tempH_test);        
    case {'hardlim'}
        %%%%%%%% Hard Limit
        H_test = hardlim(tempH_test);        
    case {'tribas'}
        %%%%%%%% Triangular basis function
        H_test = tribas(tempH_test);        
    case {'radbas'}
        %%%%%%%% Radial basis function
        H_test = radbas(tempH_test);        
        %%%%%%%% More activation functions can be added here        
end
TY=(H_test' * OutputWeight)';
output=zeros(1,size(TV.T, 2));
for i = 1 : size(TV.T, 2)
    [x, label_index_actual]=max(TY(:,i));
    output(i)=label_index_actual;
end
%output
%   TY: the actual output of the testing data
end_time_test=cputime;
TestingTime=end_time_test-start_time_test;           %   Calculate CPU time (seconds) spent by ELM predicting the whole testing data

if Elm_Type == REGRESSION
    TestingAccuracy=sqrt(mse(TV.T - TY));            %   Calculate testing accuracy (RMSE) for regression case
end

salida = zeros(size(TestingData,1),1);
if Elm_Type == CLASSIFIER
%%%%%%%%%% Calculate training & testing classification accuracy
    MissClassificationRate_Testing=0;

    for i = 1 : size(TV.T, 2)
        [x, label_index_expected]=max(TV.T(:,i));
        [x, label_index_actual]=max(TY(:,i));
        salida(i)=label_index_actual;
        if label_index_actual~=label_index_expected
            MissClassificationRate_Testing=MissClassificationRate_Testing+1;
        end
    end
    TestingAccuracy=1-MissClassificationRate_Testing/size(TV.T,2);  
end

end